# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python library imports
import sys
from time import time, localtime, strftime, gmtime
from string import strip,split,join
from string import lower
from urllib import quote, unquote
from types import *

# zope library imports
from intSet import intSet     
import Globals
from Globals import Persistent     
from Globals import HTMLFile
from Acquisition import Implicit
from DateTime import DateTime
from OFS.Traversable import Traversable
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from AccessControl import ClassSecurityInfo
from AccessControl import SecurityManagement
from AccessControl.unauthorized import Unauthorized
from ZPublisher.HTTPRequest import FileUpload
from zLOG import LOG, WARNING, ERROR, BLATHER

# zope product imports
from Products.Formulator.Form import FormValidationError

# local imports
from Attachment import Attachment, AttachmentTooLargeError
from Utility import tagRegex, getitem, intersect
from stripogram import html2safehtml
from Const import *
from FormRenderer import FormRenderer

def get_attachment_data(form_data):
    attachment = form_data.get('attachment', None)
    delete_attachment = form_data.get('delete_attachment', None)
    
    del_keys = ['delete_attachment']
    for key, value in form_data.items():
        if isinstance(value, FileUpload):
            del_keys.append(key)
    for key in del_keys:
        try:
            del form_data[key]
        except:
            pass
    return attachment, delete_attachment

def filter_form_data(form_data, field_names):    
    dict = {}
    map(dict.setdefault, field_names)
    while form_data:
        key, value = form_data.popitem()
        if key in field_names:
            dict[key] = value
    return dict

def addItem(parent, Klass, form_id, index=1, REQUEST=None, RESPONSE=None):
    def set_encodings(form_data):
        for field_name in form_data.keys():
            encoding = form_data.get("%s_encoding" % field_name, None)
            if encoding in validEncodings and encoding != MSG_ENC_PLAIN:
                s = form_data[field_name]
                form_data[field_name] = html2safehtml(s)

    form = getattr(parent.forms, form_id)
    form_data = form.validate_all(REQUEST) # throws form validation error
    field_names = map(parent.get_field_name,form.get_fields())
    
    attachment, delete_attachment = get_attachment_data(form_data)
    form_data = filter_form_data(form_data, field_names)
    
    id=parent.createId()
    thread=[]
    if not parent.root:
        map(thread.append, parent.thread)
        thread.append(int(parent.getId()))
    set_encodings(form_data)
    item=Klass(id, thread, parent.level+1, form_id, form_data).__of__(parent)
    item.edit_attachment(delete_attachment, attachment)
    parent.setItem(id, item, index)
    for t in thread:
        obj=parent.data[t]
        obj.modified=id
    if REQUEST:
        RESPONSE = RESPONSE or REQUEST.RESPONSE
    set_cookies(item, RESPONSE)
    return item

def set_cookies(entry, RESPONSE):
    gtime = gmtime(int(entry.getId()))
    glist = list(gtime)     
    glist[0] = glist[0] + 1 # add 1 year to expiry date     
    glist[1] = 12     
    glist[2] = 31     
    glist[3] = 23     
    glist[4] = 59     
    glist[5] = 59     
    glist[6] = 0     
    glist[7] = 365     
    glist[8] = 0     
    gtime = tuple(glist)     
    e = strftime('%A, %d-%b-%y %H:%M:%S GMT',gtime)     
    author = quote(entry.author)    
    RESPONSE.setCookie('_suggest_author',author,expires=e,path='/')     
    email = quote(entry.email)    
    RESPONSE.setCookie('_suggest_email',email,expires=e,path='/')     
    RESPONSE.setCookie('_suggest_notify',entry.notify,expires=e,path='/')

def guess_property_type(value):
    t = type(value)
    if t in (StringType, UnicodeType):
        return "string"
    elif IntType is t:
        return "int"
    elif LongType is t:
        return "long"
    elif t in (ListType, TupleType):
        return "lines"
    elif isinstance(value, DateTime):
        return "date"
    else:
        return "string"
        
class AbstractEntry(PropertyManager, SimpleItem):     
    """ """
    
    security = ClassSecurityInfo()

    security.setDefaultAccess("allow")
    
    _permission_defaults = ((View,              (ManagerRole, AnonymousRole)),)
    
    for permission, roles in _permission_defaults:
        security.setPermissionDefault(permission, roles)
        
    root=0

    _properties = PropertyManager._properties + (
        ({'id':'author', 'type':'string', 'mode': 'w', 'default': 'Anonymous'}),
        ({'id':'body', 'type': 'string', 'mode': 'w', 'default': ''})
        )
    form_renderer = FormRenderer()
    
    security.declarePrivate('__init__')
    def __init__(self, id, thread,level, form_id=None, form_data=None):
        self.id      =str(id)
        self._ids     =intSet()
        self.thread  =thread
        self.created =id
        self.modified=id     
        self._level =level
        self._revsub  =0     
        self._reviewed=0
        self._attachment = None
        self._reminder_count = 0
        self.author = None
        self.body = ''
        if form_id is not None:
            self.form_id = form_id
            self.edit_properties(form_data)
 
    security.declarePrivate('manage_afterAdd')    
    def manage_afterAdd(self, item, container):        
        for permission, roles in self._permission_defaults:
            try:
                self.manage_permission(permission, roles, 1)
            except:
                LOG(self.meta_type,
                    ERROR,
                    'Error setting default roles',
                    'Permission / Roles : %s/%s' % (permission, roles),
                    error=sys.exc_info())
     
    security.declarePrivate('index')
    def index(self, reindex_parent=1, reindex_children=1):
        # index this entry in the ReQuezt obj that contains it.
        oid = '/'.join(self.getPhysicalPath())
        self.catalog_object(self,oid)
        if reindex_parent and not self.aq_parent.root:
            self.aq_parent.index(reindex_children=0)
        if reindex_children:
            for item in self.entryValues():
                item.index(reindex_parent=0)

        
    security.declareProtected(View, 'getForm')
    def getForm(self):
        # """ """
        return self.forms[self.form_id]
    
    security.declareProtected(View, 'getThread')
    def getThread(self, index=None):
        if index is not None:
            return self.thread[index]
        else:
            return self.thread

    security.declarePublic('__len__')
    def __len__(self): return 1     
     
    security.declareProtected(View, '__getitem__')
    __getitem__ = getitem     
     
    security.declarePrivate('setItem')
    def setItem(self,id,obj,index=1):     
        # Make sure the object we store is not wrapped with     
        # an acquisition wrapper, since we will be wrapping     
        # it again manually in __getitem__ when it is needed.
        bobj = getattr(obj,'aq_base',obj)
        self._ids.insert(id)     
        self.data[id]=bobj

        #index the new posting
        if index:
            obj.index()

    security.declarePrivate('textToSearch')
    def textToSearch(self):
        # returns the text to search for a ZCatalog
        title = self.title_or_id()
        body = self.html2text(getattr(self, 'body', ''))
        text = [title, body]
        for item in self.entryValues():
            text.append(item.textToSearch())
        return "\n".join(text)

    security.declarePrivate('edit_properties')
    def edit_properties(self, data):
        update = {}
        for key, value in data.items():
            if not self.hasProperty(key):
                self.manage_addProperty(key, value, guess_property_type(value))
            else:
                update[key] = value
        if update:
            self.manage_changeProperties(**update)
                
        
    security.declareProtected(View, 'get_author_name')
    def get_author_name(self):
        return self.author

    security.declareProtected(View, 'date_created_string')
    def date_created_string(self,fmstr='%x'):
        # """ date when entry was created """     
        ltime = localtime(self.created)
        return strftime(fmstr,ltime)
             
    security.declareProtected(View, 'date')
    def date(self):
        """return the date of creation for indexing purposes"""
        return DateTime(self.created)

    security.declareProtected(View, 'date_sort')
    def date_sort(self):
        return DateTime(self.created).Date()
    
    security.declareProtected(View, 'body_len')
    def body_len(self,divisor=None):     
        # """ total body length of text """    
        tlen = 0
        body = getattr(self, 'body', None)
        if not body:     
            tlen = 0    
        elif ListType is type(body):
            for line in self.body:     
                tlen = tlen + len(line)
        else:
            tlen = len(body)
    
        if divisor is None:    
            if tlen == 0:    
                return ''    
            if tlen > 51200:     
                tlen = tlen / 1024     
                return str(tlen) + ' Kb'     
            else:     
                return str(tlen) + ' bytes'     
    
        if divisor < 1:    
            return tlen    
        else:    
            return tlen/divisor    
                    
    security.declarePrivate('data_map')
    def data_map(self,ids):
        result=[]
        for id in ids:
            result.append(self.data[id].__of__(self))
        return result
    
    security.declareProtected(View, 'tpId')
    def tpId(self):     
        return self.id

    security.declareProtected(View, 'title_short_or_id')
    def title_short_or_id(self):
        return "%s..." % self.title[:12] or 'Beitrag'
     
    security.declareProtected(View, 'tpURL')
    def tpURL(self):     
        return self.id     
     
    security.declareProtected(View, 'this')
    def this(self): return self     
         
    security.declareProtected(View, 'has_items')
    def has_items(self):     
        return len(self._ids)     
     
    security.declarePrivate('sub_ids')
    def sub_ids(self,ids):     
        map(ids.insert, self._ids)     
        for item in self.data_map(self._ids):     
            ids=item.sub_ids(ids)     
        return ids     
     
    security.declareProtected(View, 'desc_items')
    def desc_items(self):     
        # """ return latest list of replies """
        items = []
        entries = map(self.__getitem__,self._ids)
        reviewed = filter(lambda p: p.reviewed, entries)
        for item in reviewed:
            items.append(item)
            items.extend(item.desc_items())
        return items
     
    security.declareProtected(View, 'previous_entry_id')
    def previous_entry_id(self):
        ids = self.getCurrentResultIds()
        id = int(self.getId())
        prev_id = id
        try:
            prev_id_idx = ids.index(id) - 1
            if prev_id_idx >= 0:
                prev_id = int(ids[prev_id_idx])
        except:
            pass
        if prev_id != id:
            return prev_id
        else:
            return None
        
    security.declareProtected(View, 'next_entry_id')
    def next_entry_id(self):
        ids = self.getCurrentResultIds()
        id = int(self.getId())
        next_id = id
        try:
            next_id_idx = ids.index(id) + 1
            if next_id_idx < len(ids):
                next_id = int(ids[next_id_idx])
        except:
            pass
        if next_id != id:
            return next_id
        else:
            return None
        
    security.declareProtected(View, 'entryValues')
    def entryValues(self):
        # """ return all replies """
        return self.data_map(self._ids)
    
    security.declareProtected(View, 'suggest_title')
    def suggest_title(self):     
        # """ suggested title of reply """
        t=self.title     
        return (lower(t[:3])=='aw:') and t or 'AW: %s' % t     
     
    security.declareProtected(View, 'thread_path')
    def thread_path(self):     
        return join(map(lambda x: '/%s' % x, self.thread), '')     

    security.declarePrivate('_before_edit')
    def _before_edit(self):
        pass
    security.declarePrivate('_after_edit')
    def _after_edit(self):
        pass
    
    security.declarePrivate('_touch')
    def _touch(self):
        self.modified = int(time())
        self.index()
        self.index_html__ZCacheable_invalidate()
        
        
    security.declareProtected(EditEntries, 'edit')
    def edit(self,REQUEST=None,RESPONSE=None,reviewed='',index=1):     
        """ edit entries """     
        form = self.getForm()
        try:
            new_values = form.validate_all(REQUEST)
        except FormValidationError, error_value:
            REQUEST['error_value'] = error_value
            form_template_id = REQUEST.get('form_template_id', None)
            if form_template_id:
                form_template = getattr(self, form_template_id, None)
            else:
                form_template = None
            form_template = form_template or self.manage_edit_form
            return form_template(self, REQUEST)

        field_names = map(self.get_field_name,form.get_fields())
        new_values = filter_form_data(new_values, field_names)
        
        self._before_edit()
       
        attachment, delete_attachment = get_attachment_data(new_values)
        # attachment size is checked in form validation
        self.edit_attachment(delete_attachment, attachment)

        self.edit_properties(new_values)

        self._touch()
        
        event_id = self._after_edit()

        if not event_id:
            event_id = MESSAGE_EDIT
            
        message = messages[event_id]
        if REQUEST:
            p2 = REQUEST.get('after_edit', "%s/%s" % (self.absolute_url(), 'manage_main'))
            return self.show_message(self, REQUEST=REQUEST,
                                     title=message.title,
                                     message=message.text,
                                     action=p2)

    security.declarePrivate('edit_attachment')
    def edit_attachment(self, delete_attachment, new_attachment):
        have_new_file = (getattr(new_attachment,'filename', None))
        if delete_attachment or have_new_file:
            # delete the old file
            try:
                delattr(self,self.aq_base.attachment.filename())
            except AttributeError:
                pass
            self.attachment=''

        if have_new_file:
            # store the new file
            attachment=Attachment(new_attachment)     
            setattr(self,attachment.filename(),attachment)
            self.attachment=attachment

    # Used to display the body of the entry with the appropriate formatting    
    security.declareProtected(View, 'showBody')
    def render_body(self):
        try:
            body = self.body
            encoding = self.body_encoding
        except AttributeError:
            return self.title
        return self.form_renderer.render_text(body,encoding)
     
    # Return the plain text body of a posting suitable for mailing...
    security.declareProtected(View, 'plain_text')
    def plain_text(self):
        body = self.body
        if body:
            body_encoding = getattr(self, 'body_encoding', MSG_ENC_HTML)
        if body_encoding == MSG_ENC_HTML:
            return self.html2text(body)
        else:
            if ListType is type(body):
                return " ".join(body)
            else:
                return body
        
    security.declarePublic('getId')
    def getId(self):
        return self.id

    security.declareProtected(ModerateEntries, 'exportXML')
    def exportXML(self, xml_doc):
        """ exportXML(self) 
        create an xml representation of this entry """
        v = unicode(self.__class__.__name__, LATIN1).encode(UTF8)
        entry = xml_doc.createElement(v)
        properties = xml_doc.createElement("Properties")
        for key in dir(self):
            if key[0] == "_":
                continue # don't export private attributes
            
            p = xml_doc.createElement("Property")
            v = unicode(key, LATIN1).encode(UTF8)
            p.setAttribute("name", v)
            
            value = getattr(self, key)
            if hasattr(value, "aq_base"):
                value = value.aq_base

            v = unicode(type(value).__name__, LATIN1).encode(UTF8)
            p.setAttribute("type", v)
            # is it an object of some kind?            
            if hasattr(value, '__class__'):
                v = unicode(value.__class__.__name__, LATIN1).encode(UTF8)
                p.setAttribute("class", v)
                if isinstance(value, Attachment):
                    v = unicode("%s/%s" % (self.absolute_url(), value.filename()), LATIN1).encode(UTF8)
                    p.setAttribute("href", v)
                    text = value.filename()
                else:
                    text = str(value)
            else:
                text = str(value)

            v = unicode(text, LATIN1).encode(UTF8)
            text_node = xml_doc.createTextNode(v)
            p.appendChild(text_node)
            properties.appendChild(p)
            
        entry.appendChild(properties)
        
        if self.has_items():
            subentries = xml_doc.createElement("SubEntries")
            for subentry in self.entryValues():
                subentry_node = subentry.exportXML(xml_doc)
                subentries.appendChild(subentry_node)
            entry.appendChild(subentries)
            
        return entry
    
Globals.InitializeClass(AbstractEntry)

class Entry(AbstractEntry):
    """ """
    
    security = ClassSecurityInfo()

    security.setDefaultAccess("allow")
    
    meta_type='Entry'     
    icon   ='misc_/ReQuezt/entry'     
    _permission_defaults = AbstractEntry._permission_defaults + \
                           ((AddReplies,        (ManagerRole, ExpertRole)),
                            (AddComments,       (ManagerRole, ModeratorRole))
                            )

    _properties = AbstractEntry._properties + (({'id':'expert', 'type':'string', 'mode': 'w', 'default': ''}),
                                               ({'id':'notify', 'type': 'int', 'mode': 'w', 'default': 1}), 
                                               ({'id':'email', 'type': 'string', 'mode': 'w', 'default': ''})
                                               )
    for permission, roles in _permission_defaults:
        security.setPermissionDefault(permission, roles)
        

    manage_options = ({'label':'Edit', 'icon':icon, 'action':'manage_main', 'target':'manage_main'},
                      {'label':'View', 'icon':None, 'action':'index_html', 'target':'_BLANK'}
                      ) + SimpleItem.manage_options
    


    manage_edit_form=HTMLFile('templates/Entry_manage_edit_form', globals())
    
    security.declareProtected(ModerateEntries, 'manage_edit_form')
    if hasattr(manage_edit_form,'_setName'):
        manage_edit_form._setName('manage_edit_form')
    
    # Aliases for manage_editForm
    manage           =manage_edit_form     
    manage_main      =manage_edit_form
    manage_workspace =manage_edit_form

    security.declarePrivate('__init__')
    def __init__(self, id, thread,level, form_id=None, form_data=None):
        self.expert = None
        self.notify = None
        self.email = ''
        AbstractEntry.__init__(self, id, thread, level, form_id, form_data)
        
    security.declareProtected(View, 'this')
    def this(self): return self     

    security.declareProtected(View, 'answered')
    def answered(self):
        # needed for convenient searches
        if self.reply_count():
            return 1
        else:
            return 0
        
    def answered_by(self):
        retval = []
        for reply in self.replyValues():
            author = reply.author
            if author is not None and author not in retval:
                retval.append(author)
        return retval

    security.declareProtected(View, 'late')
    def late(self):
        # return 1 if not answered and created sooner than max_wait hours ago
        max_wait = getattr(self, 'max_wait', 72)
        if not self.answered():
            t0 = getattr(self, '_last_reminder', self.created)
            t = time() - t0
            if t > (max_wait * 3600.):
                return 1
        return 0
    
    security.declareProtected(View, 'get_expert_name')
    def get_expert_name(self):
        udm = self.get_user_data_manager()
        expert_name = udm.get_user_realname(self.expert)
        return expert_name or self.expert
    
    def get_topic_title(self):
        try:
            return self.views[self.topic].title_or_id()
        except:
            return self.topic

    security.declareProtected(View, 'reply_count')
    def reply_count(self):
        return len(self.replyValues())
    
    security.declareProtected(View, 'comment_count')
    def comment_count(self):
        return len(self.commentValues())
    
    security.declareProtected(View, 'replyValues')
    def replyValues(self):     
        # """ return all replies """
        meta_type = Reply.meta_type
        replies = filter(lambda x, meta_type = meta_type: x.meta_type == meta_type, self.entryValues())
        return replies
    
    security.declareProtected(View, 'replyValues')
    def commentValues(self):
        meta_type = Comment.meta_type
        comments = filter(lambda x, meta_type = meta_type: x.meta_type == meta_type, self.entryValues())
        return comments

    security.declareProtected(View, 'index_html')
    def index_html(self,REQUEST):
        """ """
        return self.entry_html(self, REQUEST)


    security.declarePrivate('_before_edit')
    def _before_edit(self):
        # remember current expert
        self._curr_expert = self.expert
    
    security.declarePrivate('_after_edit')
    def _after_edit(self):
        # see if expert has changed and send notifications accordingly
        # return a Message(), if we notified someone, or None
    
        if hasattr(self, '_curr_expert'):
            curr_expert = self._curr_expert
            expert = self.expert
            event_id = self._forward(curr_expert, expert)
            del self._curr_expert

            if event_id is not None:
                return event_id

    security.declarePublic('isImportant')
    def isImportant(self):
        """ """
        return str(type(self.important))
    security.declareProtected(ModerateEntries, 'remind')
    def remind(self, REQUEST=None):
        """ send (email) reminder for this entry """
        req = REQUEST or self.REQUEST
        reminder_count = self._reminder_count
        reminder_count +=1
        if self.send_email(self, entry=self, event_id=REMIND_ENTRY, user_id = self.expert, REQUEST=req,
                           reminder_count=reminder_count):
            self._reminder_count = reminder_count
            self._last_reminder = time()
            self._touch()
        else:
            LOG(self.meta_type,
                ERROR,
                'Error sending reminder',
                'Entry Id: %s\nRecipient: %s' % (self.getId(), self.expert),
                error=sys.exc_info())
        if REQUEST is not None:
            action = REQUEST.get('action')
            if action:
                RESPONSE = REQUEST.RESPONSE
                url = "%s/%s" % (REQUEST['URL2'], action)
                RESPONSE.redirect(url)
            else:
                return self.index_html(self)

    security.declareProtected(ModerateEntries, 'reminder_count')
    def reminder_count(self):
        return self._reminder_count
        
    security.declareProtected(ModerateEntries, 'last_reminder')
    def last_reminder(self):
        return getattr(self, '_last_reminder', None)
        
    security.declarePrivate('doNotify')
    def doNotify(self, entry, REQUEST, event_id=REPLY_ADDED):
        # """ sends mail to notify person being replied to """
        if self.notify and self.email:
            self.send_email(self, entry,event_id,REQUEST,email=self.email)

    security.declarePublic('cancelNotify')
    def cancelNotify(self, REQUEST):
        """ cancels email notification of replies """
        self.notify=''     
        return self.show_message(self, REQUEST=REQUEST, title='Cancelled Notification',     
                             message='You will no longer be notified of replies to this message',     
                             action=self.absolute_url()     
                            )     

    security.declarePrivate('_forward')
    def _forward(self, curr_expert, new_expert, **kw):
        if curr_expert != new_expert: # expert has changed. send email to old expert and new expert
            REQUEST = self.REQUEST
            udm = self.get_user_data_manager()
            if curr_expert:
                event_id = MOD_FORWARD_GONE
                email = udm.get_user_email(curr_expert)
                self.send_email(self, entry=self, event_id=event_id, REQUEST=REQUEST, email=email, **kw)
            if new_expert:
                event_id = MOD_FORWARD_NEW
                email = udm.get_user_email(new_expert)
                self.send_email(self, entry=self, event_id=event_id, REQUEST=REQUEST, email=email, **kw)
            self.expert = new_expert
            self._touch()
        else:
            event_id = None
        return event_id
    
    security.declareProtected(EditEntries, 'forward')
    def forward(self, form_id, information='', REQUEST=None):
        """ forward message to new expert. Update self.expert and send notification to new expert. """
        form = self.forms[form_id]
        try:
            data = form.validate_all(REQUEST)
        except FormValidationError, error_value:
            REQUEST['error_value'] = error_value
            return self.entry_forward_form(self, REQUEST=REQUEST)
        
        information = data['information']
        if ListType is type(information):
            information = "\n".join(information)
        forward_to = data['forward_to']
        event_id = self._forward(self.expert, forward_to, information=information)
        event_id = event_id or FORWARD
        message = messages[event_id]
        if REQUEST:
            p2 = self.absolute_url()
            return self.show_message(self, REQUEST=REQUEST, title=message.title,
                                     message=message.text,
                                     action=p2)
        else:
            return event_id
            
    security.declareProtected(AddReplies, 'addReply')
    def addReply(self, form_id, REQUEST=None,RESPONSE=None):     
        """ add a Reply """
        try:
            reply = addItem(self,Reply,form_id, REQUEST=REQUEST)
        except FormValidationError, error_value:
            REQUEST['error_value'] = error_value
            return self.add_reply_form(self, REQUEST=REQUEST)
        
        if REQUEST:
            self.doNotify(reply,REQUEST)
            
            moderators = self.moderators_list(include_names=0)
            user_id = SecurityManagement.getSecurityManager().getUser().getUserName()
            if not user_id in moderators:
                moderators.append(user_id)
                
            self.send_email(self, entry=reply,event_id=MOD_REPLY_ADDED, REQUEST=REQUEST, user_id=moderators)
            self._touch()
            
            resp=REQUEST['RESPONSE']     
            p2 = self.absolute_url()
            message = messages[REPLY_ADDED]
            topic = self.topic
            return self.show_message(self, REQUEST=REQUEST, title=message.title,     
                                     message=message.text,
                                     action=p2,
                                     params={'topic':topic}
                                )
    
##     security.declareProtected(AddComments, 'add_comment_form')
##     def add_comment_form(self, REQUEST):
##         """ return reply form. SecurityManament seems to be broken or I don't grok it. So we test manually... """
##         user = SecurityManagement.getSecurityManager().getUser()
##         if not user.has_permission(AddComments, self):
##             raise Unauthorized
##         return self.aq_parent.add_reply_form(self, REQUEST)
    
    security.declareProtected(AddComments, 'addComment')
    def addComment(self, form_id, REQUEST=None,RESPONSE=None):     
        """ add a Comment """
        try:
            comment = addItem(self,Comment,form_id, REQUEST=REQUEST)
        except FormValidationError, error_value:
            REQUEST['error_value'] = error_value
            return self.add_comment_form(self, REQUEST=REQUEST)

        if REQUEST:
            self.doNotify(comment,REQUEST)
            moderators = self.moderators_list(include_names=0)
            user_id = SecurityManagement.getSecurityManager().getUser().getUserName()
            if not user_id in moderators:
                moderators.append(user_id)
            self.send_email(self, entry=comment,event_id=MOD_COMMENT_ADDED, REQUEST=REQUEST, user_id=moderators)
            self._touch()

            resp=REQUEST['RESPONSE']     
            p2 = self.absolute_url()
            message = messages[COMMENT_ADDED]
            topic = self.topic
            return self.show_message(self, REQUEST=REQUEST, title=message.title,
                                     message=message.text,
                                     action=p2,
                                     params={'topic':topic}
                                )
    def pending(self):
        reply_meta_type = Reply.meta_type
        for entry in self.entryValues():
            if entry.meta_type == reply_meta_type:
                return 0
        return 1

     
Globals.InitializeClass(Entry)

# Comment has to be in this file to stop import infinite recursion
class Reply(AbstractEntry):
    """  """     
    security = ClassSecurityInfo()

    security.setDefaultAccess("allow")
    
    meta_type  ='Reply'     
    icon       ='misc_/ReQuezt/reply'

    _permission_defaults = AbstractEntry._permission_defaults
    
    manage_options = ({'label':'Edit', 'icon':icon, 'action':'manage_main', 'target':'manage_main'},
                      ) + SimpleItem.manage_options

    security.declareProtected(View, 'index_html')
    def index_html(self,REQUEST):     
        """ """    
        return self.reply_html(self,REQUEST)

    def get_author_name(self):
        udm = self.get_user_data_manager() # get user_data_manager from requezt (acquired)
        author_name = udm.get_user_realname(self.author)
        return author_name or self.author
     
Globals.InitializeClass(Reply)

class Comment(AbstractEntry):
    security = ClassSecurityInfo()

    security.setDefaultAccess("allow")
    
    _permission_defaults = AbstractEntry._permission_defaults
    for permission, roles in _permission_defaults:
        security.setPermissionDefault(permission, roles)
        
    
    meta_type = 'Comment'
    icon      = 'misc_/ReQuezt/comment'
    manage_options = ({'label':'Edit', 'icon':icon, 'action':'manage_main', 'target':'manage_main'},
                      ) + SimpleItem.manage_options
    
    security.declareProtected(View, 'index_html')
    def index_html(self,REQUEST):     
        """ """    
        return self.comment_html(self,REQUEST)
    
    def get_author_name(self):
        udm = self.get_user_data_manager() # get user_data_manager from requezt (acquired)
        author_name = udm.get_user_realname(self.author)
        return author_name or self.author


Globals.InitializeClass(Comment)
