# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports 
from types import *

# zope class imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.ZopeGuards import guarded_getattr
from OFS.ObjectManager import REPLACEABLE
from OFS.PropertyManager import PropertyManager
from ZPublisher.HTTPRequest import FileUpload
# zope utility function imports
from StructuredText import html_with_references

# modules imports
import translations
from Permissions import View
from Utility import html_quote, url_quote, stx_min, stx_max, getTemplateFile
from stripogram import html2safehtml, html2text
from Const import *

class FormRenderer:
    security = ClassSecurityInfo()
    _default_valid_tags        = ('b', 'a', 'i', 'br', 'p', 'h3', 'ul', 'li', 'font', 'br')

    security.declareProtected(View, 'vacuum_request')
    def vacuum_request(self, REQUEST):
        """ remove FileUploadFields from REQUEST """
        for dict in [REQUEST.form, REQUEST.other]:
            for key, value in dict.items():
                if isinstance(value, FileUpload):
                    del dict[key]
        return REQUEST
            
        
    security.declareProtected(View, 'render_static')
    def render_static(self, field, obj, default="-"):
        """ render (Formulator) field 'field' as string, i.e. without an <input> tag.
            use this to display data in a report
        """
        def allowed_img_size(width, height, max_width=max_img_width, max_height=max_img_height):
            factor = 1
            if not (width or height):
                return 0, 0
            
            if width > height:
                if width > max_width:
                    factor = max_width / float(width)
            else:
                if height > max_height:
                    factor = max_height / float(height)
            return tuple(map(int, [width * factor, height * factor]))
        
        # if obj is an acquisition wrapper, get an unwrapped version in base_obj
        # still, we need to keep the wrapped object around, because some methods
        # like 'absolute_url()' only work on wrapped objects
        if hasattr(obj, 'aq_base'):
            base_obj = obj.aq_base
        else:
            base_obj = obj
            
        image_mime_types = ["image/gif", "image/jpg", "image/jpeg", "image/png"]
        field_name = self.get_field_name(field)
        if DictType is type(base_obj):
            field_value = base_obj.get(field_name, None)
        elif isinstance(base_obj, PropertyManager) and base_obj.hasProperty(field_name):
            field_value = base_obj.getProperty(field_name)
        else:
            field_value = getattr(base_obj, field_name, None)
        if field_value is None:
            return ''
    
        meta_type = field.meta_type
        try:
            if meta_type == "StringField":
                retval = html_quote(field_value)
            elif meta_type == "PasswordField":
                retval = "*" * 8
            elif meta_type == "EmailField":
                retval = """<a href="mailto:%s">%s</a>""" % (url_quote(field_value), html_quote(field_value))
            elif meta_type == "CheckBoxField":
                if field_value:
                    retval = "Yes"
                else:
                    retval = "No"
            elif meta_type == "IntegerField":
                retval =  str(field_value)
            elif meta_type == "RangedIntegerField":
                retval =  str(field_value)
            elif meta_type == "FloatField":
                retval =  str(field_value)
            elif meta_type == "TextAreaField":
                attr = "%s_encoding" % field_name
                encoding = getattr(obj, attr, None)
                if encoding:
                    retval = self.render_text(field_value, encoding)
                else:
                    retval =  html_quote(field_value)
            elif meta_type == "LinesField":
                retval = html_quote(" ".join(field_value))
            elif meta_type == "ListField":
                items = field.get_value("items")
                retval = None
                for value in items:
                    if TupleType is type(value) or ListType is type(value):
                        if value[1] == field_value:
                            retval = value[0]
                            break
                    else:
                        break
                retval = retval or html_quote(str(field_value))
            elif meta_type == "RadioField":
                retval =  html_quote(str(field_value))
            elif meta_type == "DateTimeField":
                retval =  html_quote(str(field_value))
            elif meta_type in ["MultiCheckBoxField", "MultiListField"]:
                retval =  html_quote("\n".join(field_value))
            elif meta_type == "LinkField":
                retval = "<a href='%s' target='_BLANK'>%s</a>" % (field_value, html_quote(field_value))
            elif meta_type == "FileField":
                base_url = obj.absolute_url()
                if hasattr(field_value, 'filename'):
                    content_type = field_value.getContentType()
                    content_type = content_type.split(";")[0]
                    if content_type in image_mime_types:
                        orig_width = getattr(field_value, 'width', 0)
                        orig_height = getattr(field_value, 'height', 0)
                        width, height = allowed_img_size(orig_width, orig_height)
                        if width and height:
                            size_attr = " width='%i' height='%i'" % (width, height)
                        else:
                            size_attr = ""
                        retval = "<img src='%s/%s'%s border='0' alt='%s'>" % (base_url, field_value.filename(), size_attr, field_value.filename())
                        if orig_width > width:
                            retval ="<a href='%s/%s' target='_BLANK'>%s</a>" % (base_url, field_value.filename(), retval)
                    else:
                        retval = "<a href='%s/%s'>%s (%s Kb)</a>" % (base_url, field_value.filename(), field_value.filename(), field_value.file_kbytes())
                else:
                    retval = html_quote(str(field_value))
            else:
                retval = "%s (unknown meta_type %s for field %s)" % (html_quote(field_value), meta_type)
        except:
            raise
    ##            retval = "Error rendering field value for field %s (raw_value:%s" % (html_quote(field.getId()), html_quote(field_value))
        if ListType is type(retval):
            retval = join(retval)
        return retval or default
    
    security.declareProtected(View, 'render_form')
    render_form = getTemplateFile("templates/render_form", globals())
    render_form.__replaceable__ = REPLACEABLE
    
    security.declareProtected(View, 'render_report')
    render_report = getTemplateFile("templates/render_report", globals())
    render_report.__replaceable__ = REPLACEABLE
    
    security.declareProtected(View, 'get_error_for_field')
    def get_error_for_field(self, errors, field):
        """ from a list of error objects, 'errors',
            return the error for field 'field', or None 
            if no such error exists
        """
        for error in errors:
            if error.field_id == field.id:
                return error
        return None
    security.declareProtected(View, 'get_field_name')
    def get_field_name(self, field):
        if field.has_value('alternate_name'):
            return field.get_value('alternate_name')
        else:
            return field.id
    
    security.declareProtected(View, 'get_edit_attr')    
    def get_edit_attr(self, obj, attr, default=''):
        if DictType is type(obj):
            value = obj.get(attr, default)
        elif hasattr(obj, attr):
            base_obj = getattr(obj, 'aq_base', obj)
            if hasattr(base_obj, attr):
                value = guarded_getattr(obj, attr)
                if hasattr(value, 'filename'):
                    # handle attachments
                    value = value.filename()
                elif callable(value):
                    value = value()
            else:
                value = default
        else:
            value = default
        
        return value
    
    security.declareProtected(View, 'get_filtered_fields_in_group')    
    def get_filtered_fields_in_group(self, form, group, obj, omit_empty_fields=0):
        
        def field_is_valid_for_obj(field, get_field_name=self.get_field_name, obj=obj):
            if getattr(obj, get_field_name(field), None):
                return 1
            return 0

        fields = form.get_fields_in_group(group)
        fields = filter(lambda x: not x.get_value('hidden'), fields)
        if omit_empty_fields:
            fields = filter(field_is_valid_for_obj, fields)
        return fields

    # This renders the raw document text into stuff that can be displayed with a dtml-var
    # This method is overidable by placing a script/method called 'render' in the ReQuezt object
    
    security.declareProtected(View, 'render_text')
    def render_text(self, text, format):
        format = format.upper()
        if not text:
            return ''
        if StringType is type(text):
            text = text.split("\n")
        if format==MSG_ENC_PLAIN:
            return "<br />\n".join(map(html_quote,text))
        elif format==MSG_ENC_HTML:
            return "\n".join(text)
        elif format==MSG_ENC_STX:
            text = "\n".join(text).replace('\r', '')
            text = html_with_references(text, level=3)[stx_min:stx_max]
            return text
        return 'Invalid Format!'
    
    render_text__replaceable__ = REPLACEABLE
    
    security.declareProtected(View, 'striptags')
    def striptags(self,s):
        # ''' removes char sequences that invalidate RSS syntax'''     
        return tagRegex.sub("",s)

    security.declareProtected(View, 'html2text')
    def html2text(self,s):
        return html2text(s)
     
    security.declareProtected(View, 'html2safehtml')
    def html2safehtml(self,text):
        valid_tags = getattr(self, 'valid_tags', self._default_valid_tags)
        return strip(html2safehtml(text,self.getProperty('valid_tags',)))

    security.declareProtected(View, 'get_translation')
    def get_translation(self, phrase, lang=translations.DEFAULT_LANG):
        """ translate(self, phrase, lang=DEFAULT_LANG):
            return translation for phrase 'phrase' in to language 'lang' or default. 
            might be an interface to a i18n and l10n system like ZBabel. Right now doesn't do a lot...
            this method might move somewhere else (into ReQuezt)"""
        s = translations.get_translation(phrase, lang)
        return s
    
    security.declareProtected(View, 'get_group_title')
    def get_group_title(self, group_id, report=0):
        """ get a group title. split at '|'. return first part for form,
        second, if present, for reports """
        titles = group_id.split('|')
        if len(titles) == 1:
            titles.append(titles[0])
        if report:
            return titles[1]
        else:
            return titles[0]
        
        
InitializeClass(FormRenderer)
