# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports
import urllib
from types import ListType
from time import time

# zope imports
from Globals import DTMLFile, InitializeClass, ImageFile
from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from Products.PageTemplates import ZopePageTemplate
from OFS.Image import Image
from OFS.Folder import Folder
from OFS.ObjectManager import REPLACEABLE
from DateTime import DateTime

# product import
from Products.OrderedFolder.SubObjectTypeSupport import SubObjectTypeSupport

# own imports
from Utility import addImage
from Utility import addPythonScript
from Utility import getTemplateFile
from Utility import initialize_permissions
from Permissions import View, ExpertRole, ModeratorRole

manage_addReQueztViewForm = DTMLFile('manage_addReQueztViewForm', globals())
manage_addReQueztTopicForm = DTMLFile('manage_addReQueztTopicForm', globals())

def manage_addReQueztView(self, id, title='', title_short='', image=None, description_data=None, hidden=0, allowed_roles=None, REQUEST=None):
    '''manage_addReQueztView(self, id, title='', REQUEST=None) --> add ReQueztView to a ReQueztViewFolder;
       return None or HTML'''

    view = ReQueztView(id, title, title_short, image, description_data, hidden)
    self._setObject(view.id, view)
    ob = self._getOb(view.id).__of__(self)
    initialize_permissions(ob, allowed_roles)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')

def manage_addReQueztTopic(self, id, title='', title_short='', image=None, description_data=None, hidden=0, allowed_roles=None, REQUEST=None):
    '''manage_addReQueztTopic(self, id, title='', REQUEST=None) --> add ReQueztTopic to a ReQueztViewFolder;
       return None or HTML'''

    topic = ReQueztTopic(id, title, title_short, image, description_data)
    self._setObject(topic.id, topic)
    ob = self._getOb(topic.id).__of__(self)
    initialize_permissions(ob, allowed_roles)
    if REQUEST:
        return REQUEST.RESPONSE.redirect('../../manage_main')

class ReQueztView(Folder, SubObjectTypeSupport):
    ''' ReQuezt View '''
    meta_type = 'ReQuezt View'
    icon = 'misc_/ReQuezt/ReQueztView'

    security = ClassSecurityInfo()
    security.setDefaultAccess('allow')
    _properties=({'id':'title', 'type': 'string', 'mode': 'w'},
                 {'id':'title_short', 'type':'string', 'mode' : 'w'},
                 {'id':'hidden', 'type':'boolean', 'mode': 'w', 'default': 0}
                 )

    def __init__(self, id, title='', title_short='', image=None, description_data=None, hidden=1):
        '''__init__(self, id, title='') --> initialize the object; return None'''
        
        self.id = str(id)
        self.title = title
        self.title_short = title_short or self.id
        self.hidden = hidden

        # initialize the properties
        for property in self._properties:
            try:
                prop_id = property['id']
                prop_value = getattr(self, prop_id, property['default'])
                setattr(self, prop_id, prop_value)
            except: pass

        # add a page template for a customized description
        if description_data:
            ZopePageTemplate.manage_addPageTemplate(self, 'description', 'Beschreibung', description_data)
        
    security.declareProtected(View, 'image')
    image = ImageFile('www/ReQueztView.png', globals())
    image__replaceable__ = REPLACEABLE
    
    security.declareProtected(View, 'description')
    description = getTemplateFile('templates/ReQueztView_description', globals())
    description.__replaceable__ = REPLACEABLE
    
    security.declareProtected(View, 'default_filter')
    def default_filter(self):
        # """ return a default filter. usually we only want to see <Entry>-objects.
        #     Experts only want to see messages assigned to their user-id. 
        #     Also, for performance reasons, we don't want to see entries older than
        #     30 days. (Sorting is more expensive). TODO: This should be configurable !
        filter = {'meta_type': 'Entry'}
        user = getSecurityManager().getUser()
        roles = user.getRolesInContext(self)
        if ExpertRole in roles and not ModeratorRole in roles:
            filter['expert'] = user.getUserName()
        else:
            filter['expert'] = ''
        age_treshold = 30 * 86400
        filter['date'] = int(time() - age_treshold)
        filter['date_usage'] = 'range:min'
        return filter
        
    security.declareProtected(View, 'getFilter')
    def getFilter(self):
        return self.default_filter()
    getFilter__replaceable__ = REPLACEABLE

    security.declareProtected(View, 'getFilterUrlParams')    
    def getFilterUrlParams(self):
        filter = self.getFilter()
        if filter:
            url_params = map(lambda x, y, q=urllib.quote: "%s=%s" % (q(str(x)), q(str(y))), filter.keys(), filter.values())
            url_params = "&".join(url_params)
        else:
            url_params = ''
        return url_params
                
    security.declareProtected(View, 'index_html')
    def index_html(self, REQUEST=None):
        """ index_html(self, REQUEST) """
        session_data = self.getSessionData()
        id = self.getId()
        if session_data.get('active_view', None) != id:
            session_data['start'] = 0
        session_data['active_view'] = id
        REQUEST.set('filter', self.getFilter())
        parent = self.aq_parent.__of__(self)
        method = parent.index_html
        if hasattr(method, '__of__'):
            return method.__of__(self)(REQUEST)
        else:
            return method(REQUEST)

    security.declareProtected(View, '__getitem__')
    def __getitem__(self, key):
        v = getattr(self, key, None)
        if v is not None: return v
        return self.aq_parent[key].__of__(self)
    
InitializeClass(ReQueztView)

class ReQueztTopic(ReQueztView):
    meta_type = 'ReQuezt Topic'
    icon = 'misc_/ReQuezt/ReQueztTopic'
    
    security = ClassSecurityInfo()
    security.setDefaultAccess('allow')
    
    def __init__(self, id, title='', title_short='', image=None, description_data='', hidden=0):
        '''__init__(self, id, title='') --> initialize the object; return None'''
        ReQueztView.__init__(self, id, title, title_short, image, description_data, hidden)
        
    security.declareProtected(View, 'getFilter')
    def getFilter(self):
        filter = ReQueztView.getFilter(self)
        filter.update({'topic':self.getId()})
        return filter
    
    getFilter__replaceable__ = REPLACEABLE
